# Nagios::Config.pm
# 
# Parses nagios-like config files
#
#Copyright (C) 2004 Tekco Management Group, LLC
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package Nagios::Config;
$VERSION = 0.01;

use Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw();
@EXPORT_OK = qw();

use strict;
use Carp;
use Data::Dumper;
use Fcntl qw(:flock);

# globals
use vars qw($VERSION $DEBUG $error);

$DEBUG = 0;

#----------------------------------------
# Constructor
#----------------------------------------
sub new {
  my $class = shift;
  my $this  = {};
  bless($this, $class);
  $this->_initialize(@_) || return(undef);
  return($this);
}

sub _initialize($) {
  my $this  = shift;

  $this->{cfg} = {};

  if(@_ == 1) {
    $this->{filename} = shift;
  } else {
    my %opts  = @_;

    $DEBUG            = $opts{DEBUG}     if $opts{DEBUG};
    $this->{filename} = $opts{filename}  if $opts{filename};
    $this->{path}     = $opts{path}      || [ '/etc/', '/usr/local/etc/' ];
    $this->set_defaults($opts{defaults}) if $opts{defaults};
    $this->set_required($opts{required}) if $opts{required};
  }

  return($this->parse());
}

#----------------------------------------
# Public Methods
#----------------------------------------
sub parse($) {
  my $this = shift;
  my $file;
  
  if(!$this->{filename}) {
    $error = "No config filename given";
    return(0);
  }

  # find the config file if we need to
  if(substr($this->{filename}, 0, 1) eq '/') {
    $file = $this->{filename};
  } else {
    foreach my $path (@{ $this->{path} }) {
      print STDERR "Searching $path for " . $this->{filename} . "\n" if $DEBUG;
      if(-e "$path/" . $this->{filename}) {
        print STDERR "  Found in $path\n" if $DEBUG;
        $file = "$path/" . $this->{filename};
        last;
      }
    } 
  }

  if(!$file || !-e $file) {
    $error = "Config file " . $this->{filename} . " not found";
    return(0);
  }

  # open the config file
  open(CONF, $file)
    or $error = "Could not open file $file for reading: $!", return(0);

  # lock it
  flock(CONF, LOCK_EX)
    or $error = "Could not lock conf file: $!", return(0);

  # parse it
  my $current_define = undef;
  my $define_data    = undef;
  my $line_number    = 0;
  
  while(my $line = <CONF>) {
    chomp($line);
    $line_number++;

    # strip comments
    $line =~ s/#.*$//g;

    # skip blank lines
    if($line =~ /^\s*$/) {
      next;
    }

    print STDERR "line: [$line]\n" if $DEBUG;

    if($line =~ /^\s*define\s+(\S+)\s+{\s*$/) {
      $current_define = $1;
      $define_data = {};
      print STDERR "  Started define $current_define\n" if $DEBUG;
    } elsif($line =~ /^\s*}\s*$/) {
      if(!$current_define) {
        die("Syntax error on line $line_number of $file near \"$line\"\n");
      }
      if(!exists($this->{cfg}{$current_define})) {
        $this->{cfg}{$current_define} = [];
      }
      push(@{ $this->{cfg}{$current_define} }, $define_data);
      print STDERR "  Finished define $current_define\n" if $DEBUG;
      $current_define = undef;
      $define_data    = undef;
    } elsif($line =~ /^\s*([^=:\s]+)\s*(?:=>|[=:\s])+\s*(.*)\s*$/) {
      my($key, $val) = ($1, $2);

      # strip quotes
      if(substr($val, 0, 1) eq '"' && substr($val, -1, 1) eq '"') {
        $val =~ s/^"//;
        $val =~ s/"$//;
      }
      
      if($current_define) {
        print STDERR "  Adding to define $current_define: $key => $val\n" if $DEBUG;
        $define_data->{$key} = $val;
      } else {
        print STDERR "  Setting $key => $val\n" if $DEBUG;
        $this->{cfg}{$key} = $val;
      }
    }
  }

  print STDERR "cfg: [" . Dumper($this->{cfg}) . "]\n" if $DEBUG;

  # check required
  foreach my $require (@{ $this->{required} }) {
    if(!$this->param($require)) {
      $error = "Error: config variable $require is required";
      return(0);
    }
  }

  return(1);
}

sub set($$$) {
  my $this = shift;
  my($key, $val) = @_;

  $this->{cfg}{$key} = $val;
}

sub param($$;$$) {
  my $this = shift;
  my($key, $match_key, $match_val) = @_;

  if(ref($this->{cfg}{$key}) eq 'ARRAY') {
    if($match_key || $match_val) {
      my @return;
      foreach my $entry (@{ $this->{cfg}{$key} }) {
        my $matched = 1;
        if($match_key && !exists($entry->{$match_key})) {
          $matched = 0;
        }
        if($match_val) {
          my $matched_val = 0;
          foreach my $entry_key (keys(%$entry)) {
            if($match_val eq $entry->{$entry_key}) {
              $matched_val = 1;
            }
          }
          if(!$matched_val) {
            $matched = 0;
          }
        }

        if($matched) {
          push(@return, $entry);
        }
      }

      if(wantarray()) {
        return(@return);
      } else {
        return($return[0]);
      }
    } else {
      if(wantarray()) {
        return(@{ $this->{cfg}{$key} });
      } else {
        return($this->{cfg}{$key}[0]);
      }
    }
  } else {
    return($this->{cfg}{$key});
  }
}

sub set_defaults($$) {
  my $this = shift;
  my($defaults) = @_;

  while(my($key, $val) = each(%$defaults)) {
    if(!$this->param($key)) {
      $this->set($key, $val);
    }
  }
}

sub set_required($$) {
  my $this = shift;
  my($required) = @_;

  $this->{required} = $required;
}

sub AUTOLOAD {
  my $this = shift;
  use vars qw($AUTOLOAD);

  my $param = $AUTOLOAD;
  $param =~ s/.*:://;

  return if $param eq 'DESTROY';

  if(wantarray()) {
    return($this->param($param, @_));
  } else {
    my $ret = $this->param($param, @_);
    return($ret);
  }
}

sub error() {
  my $this = shift;

  return($error);
}

1;
